#
# This example uses the old approach to active tension modelling
# to simulate the active contraction and relaxation of a single 
# element, tri-cubic Hermite, rectangular cartesian cube.
#

set echo;

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

# Set the directory to store results
$output = ".";
if (! -d $output) {
    mkdir $output,0700;
}

# Set up the geometry 1*1*1 element 4*4*4mm tri-cubic Hermite basis, 
# 0 fibre angle
fem define parameter;r;$example/minimal;
fem define coordinates;r;$example/emech;
fem define node;r;$example/emech-1x1x1-cubic;
# Need a tri-cubic Hermite for geometry (and bi-cubic Hermite for faces)
# and a tri-linear for fibres (and bi-linear for faces)
# and a auxilary basis for pressure
fem define base;r;$example/emech-cubic-linear;
fem define element;r;$example/emech-1x1x1-cubic;
# 0deg fibre angles everywhere, using a tri-linear interpolation, and
# need to define sheets etc...
fem define fibre;r;$example/emech-0deg;
fem define element;r;$example/emech-1x1x1-linear fibre;

# Need to calculate the undeformed nodal derivatives
fem update node derivative 1 linear;
fem update node derivative 2 linear;
fem update node derivative 3 linear;

# Node/element groups to set BC's
fem group node external as bdy_nodes;
fem group node 1,10,19,28 as left_end;
fem group node 2,11,20,29 as right_end;
fem group node left_end as corners;
fem group node 29 as wall_x;
fem group element 1 as ALL_ELEMENTS;

# Set up the mechanics
fem define equation;r;$example/mech-cubic;
fem define material;r;$example/mech-cubic;
# fix the corners group and allow wall_x group to slide in x-axis
fem define initial;r;$example/mech-cubic-fixed;
fem define solve;r;$example/newton;

# Export the undeformed geometry
fem export node;$output/slab as slab;
fem export element;$output/slab as slab;
# ... and define the deformed field ...
fem export node;$output/deformed_0 field as slab;
fem export element;$output/deformed field as slab;

# Loop through the activation levels desired, creating the ipacti
# file to set the current activation level and solve the finite
# elasticity model.
$counter = 0;
for $activation_level (0..4) {
    if ($activation_level > 2) {
	$counter++;
	$actn = 2 - $counter;
    } else {
	$actn = $activation_level;
    }
    $actn *= 0.1;
    # Create the IPACTI file
  CORE::open(IPACTI,">tmp_ipacti_file.ipacti") || die "Cannot open temporary ipacti file: $!\n";
    print IPACTI " CMISS Version 1.21 ipacti File Version 2\n";
    print IPACTI " Heading: Temp. ipacti file for activation level of $actn\n";
    print IPACTI " \n";
    print IPACTI " Specify type of contraction model [2]:\n";
    print IPACTI "   (1) SS tension-length-Ca relation (OLD)\n";
    print IPACTI "   (2) Hunter/McCulloch/ter Keurs (coupled to cell/grid model)\n";
    print IPACTI "   (3) Unused\n";
    print IPACTI "   (4) Outer hair cell stiffness\n";
    print IPACTI "    1\n";
    print IPACTI " Enter max isometric tension at ext.ratio=1 (Tref) [100kPa]: 0.10000D+03\n";
    print IPACTI " Enter non-dimensional slope parameter (beta) [1.45]: 0.14500D+01\n";
    print IPACTI " Enter c50 for [Ca]i saturation curve (0<c<1) [0.5]: 0.50000D+00\n";
    print IPACTI " Enter Hill coeff. for [Ca]i saturation curve (h) [3.0]: 0.30000D+01\n";
    print IPACTI " Enter max [Ca]i (Ca_max) [1]: 0.10000D+01\n";
    print IPACTI " Specify whether the initial calcium level [Ca]i is [1]:\n";
    print IPACTI "  (1) Constant spatially\n";
    print IPACTI "  (2) Piecewise constant (defined by elements)\n";
    print IPACTI "  (3) Defined by Gauss points\n";
    print IPACTI "   1\n";
    print IPACTI " Enter initial calcium level [Ca]i [0]: $actn\n";
    print "Calcium = $actn\n";
  CORE::close IPACTI;
    # Define the current level of activation
    fem define active;r;tmp_ipacti_file;
    unlink "tmp_ipacti_file.ipacti";
    # Solve the current step
    fem solve increment 0.0 iterate 20 error 0.001;
    # Export the strains
    fem update gauss strain fibre components;
    fem export gauss;$output/gauss_strain_$activation_level yg as slab;
    # and stresses
    fem update gauss stress fibre components;
    fem export gauss;$output/gauss_stress_$activation_level yg as slab;
    # and nodes
    fem export node;$output/deformed_$activation_level field as slab;
}

# Make sure that we have constant volume
fem list element total
fem list element deformed total
