# make_flypath_movie changes the view in graphics window 1 from initial_position
# to final position in the given number_of_iterations, adding a frame to the current
# movie at each step. Notes:
# - If the optional $movie_name is supplied, it will be created and ended wholly within
#   this subroutine.
# - If the optional $central_position is supplied, the view will be interpolated
#   quadratically from initial to final positions via it, useful for controlling the
#   speed of transition at either end. If omitted, linear interpolation is used.
# - the optional $iterator_function allows custom behaviour at each iteration. This
#   user-defined function should be of the following form, taking the $iteration_number
#   as its only argument:
#
# $my_iterator_function = sub {
#   my $iteration_number = shift;
#
#   # Your stuff here
# }
#
# Call this function in the following way:
# @start = (-200.672, -896.653, 211.577, -146.565, 28.517, 35.0058, 0.0212965, 0.186225, 0.982276, 40);
# @finish = (175.843, 447.448, 890.264, -19.8528, 23.3261, 70.5884, -0.36608, 0.85929, -0.35722, 15.7865);
# Note the 10 finish position values (and similarly start) are taken from gfx list window commands:
#   gfx modify window 1 view perspective eye_point 175.843 447.448 890.264 interest_point -19.8528 23.3261 70.5884 up_vector -0.36608 0.85929 -0.35722 view_angle 15.7865 near_clipping_plane 9.43421 far_clipping_plane 3371.47 relative_viewport ndc_placement -1 1 2 2 viewport_coordinates 0 0 1 1
# make_flypath_movie("my_movie_name", 50, \@start, \@finish, , $my_iterator_function);
#
# Note from the above that the @central_position has been omitted by putting nothing
# between the commas, since it is followed by another parameter that is used.
#
sub quadratic_flypath_movie
{
	my $movie_name = shift; # Optional; put nothing between commas if not supplied
	my $number_of_iterations = shift;
	my $initial_position = shift;
	my $final_position = shift;
	my $central_position = shift; # Optional
	my $iterator_function = shift; # Optional
	my $iteration_number;
	my $xi;

	if ($movie_name)
	{
		print "Creating flypath movie $movie_name\n";
		#gfx movie create $movie_name rle24_sgi_movie3 window 1;
	} else {
		print "Adding flypath to current movie\n";
	}
	for ($iteration_number = 0 ; $iteration_number < $number_of_iterations ; $iteration_number++)
	{
		$xi = $iteration_number / ($number_of_iterations - 1);
		if ($central_position)
		{
			# quadratic interpolation
			@position = map {$${initial_position}[$_] * (2.0*$xi*$xi - 3.0*$xi + 1.0) + $${central_position}[$_] * (-4.0*$xi*$xi + 4.0*$xi) + $${final_position}[$_] * (2.0*$xi*$xi - $xi)} 0..$#{$initial_position};
		} else {
			# linear interpolation
			@position = map {$${initial_position}[$_] * (1.0 - $xi) + $${final_position}[$_] * $xi} 0..$#{$initial_position};
		}
		print "Position $iteration_number (xi = $xi): @position\n";
		gfx modify window 1 view eye_point $position[0] $position[1] $position[2] interest_point $position[3] $position[4] $position[5] up_vector $position[6] $position[7] $position[8] view_angle $position[9];
		if ($iterator_function)
		{
			&$iterator_function($iteration_number);
		}
		gfx movie add_frame;
	}
	if ($movie_name)
	{
		#gfx movie end;
	}
}
